#include <cstdio>
#include <string>

//#include "DIGFileSnarkRec.h"
#include <DIGFileSnark/DIGFileSnarkRec.h>

#define DIGFILESNARKREC_DEBUG_LEVEL 0

const char* DIGFileSnarkRec::TypeStr = "SNARK05 recfil";
const char* DIGFileSnarkRec::SchemaStr = "DIGFileSnarkRec.xsd";

DIGFileSnarkRec::DIGFileSnarkRec()
{
}	


DIGFileSnarkRec::~DIGFileSnarkRec()
{
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Writing
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Open for writing
///////////////////////////////////////////////////////////////////////////////

int DIGFileSnarkRec::Open(const char* pFileName, RecFileMH* pMainHeader)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::Open\n");
#endif

  return Open(
    pFileName,                  // file name
    pMainHeader->Title.Get(),   // title
    pMainHeader->Dimensions,    // dimensions
    pMainHeader->Sampling,      // sampling
    pMainHeader->Comment.Get()  // comment 
  );
}

int DIGFileSnarkRec::Open(
  const char*   pFileName,     // File Name
  const char*   pProjName,     // Projection Name
  unsigned int  pNoOfElements, // NELEM
  double        pSampling,     // PIXSIZ
  const char*   pComments      // Comments
) 
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::Open\n");
#endif

  PhantomPresent = false;
  RecPressent = false;

  DIGDimensions  dim;
  DIGSampling    sampX;
  DIGSampling    sampY;
  DIGSampling    sampZ = { 0.0, 0.0, 0.0 };

  dim.x = pNoOfElements;
  dim.y = pNoOfElements;
  dim.z = 1;

  sampX.x = pSampling;
  sampX.y = 0.0;
  sampX.z = 0.0;

  sampY.x = 0.0;
  sampY.y = pSampling;
  sampY.z = 0.0;

  return DIGFile::Open(
    pFileName,            // File Name
    SchemaStr,            // Schema
    pProjName,            // Title
    TypeStr,              // Type,
    1,                    // Chanels
    DIGValueType_REAL,    // ValueType
    DIGDataType_DOUBLE,   // DataType
    DIGDataFormat_BINARY, // DataFormat
    DIGGrid_SC,           // Grid
    DIGBasis_VORONOI,     // Basis
    DIGUnit_UNSPECIFIED,  // Unit
    &dim,                 // Dimensions
    &sampX,               // SamplingX
    &sampY,               // SamplingY
    &sampZ,               // SamplingZ
    pComments,            // Comments
    ""                    // Other Unit
  );
};

///////////////////////////////////////////////////////////////////////////////
// Updating main header after opening file to write
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Appending phantom
///////////////////////////////////////////////////////////////////////////////

int DIGFileSnarkRec::AppendPhanom(RecFilePhanomH* pPhanomHeader, double* pData)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::AppendPhanom\n");
#endif

  return AppendPhantom(pPhanomHeader->Name.Get(), pPhanomHeader->Comment.Get(), pData);
}

int DIGFileSnarkRec::AppendPhantom(const char* pPhanomName, const char* pComment, double* pData)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::AppendPhanom\n");
#endif

	// writes a phantom array into the file
	// should only be used as first array set

  if(RecPressent) {

#if DIGFILESNARKREC_DEBUG_LEVEL > 0
  printf("** Error: Reconstructions already in the file\n");
#endif
    return -1; // reconstructions already in the file
  }

  if(PhantomPresent) {
#if DIGFILESNARKREC_DEBUG_LEVEL > 0
  printf("** Error: Phantom already in the file\n");
#endif
    return -1; // phantom already in the file
  }

  if(AppendArraySet("PHAN", pPhanomName, "", pComment) != 0) {
		return 1;
  }

  if(AppendArray(0, "", pData) != 0) {
		return 2;
  }

  PhantomPresent = true;

	return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Appending reconstruction set
///////////////////////////////////////////////////////////////////////////////

int DIGFileSnarkRec::AppendRecSet(RecFileRecSetH* pRecSetHeader)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::AppendRecSet\n");
#endif

  return AppendRecSet(pRecSetHeader->AlgName.Get(), pRecSetHeader->Name.Get(), pRecSetHeader->Comment.Get());
}

int DIGFileSnarkRec::AppendRecSet(const char* pAlgName, const char* pName, const char* pComment)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::AppendRecSet\n");
#endif

  return AppendArraySet(pAlgName, pName, "", pComment);
}

///////////////////////////////////////////////////////////////////////////////
// Appending reconstruction
///////////////////////////////////////////////////////////////////////////////

int DIGFileSnarkRec::AppendRec(RecFileRecH* pRecHeader, double* pData)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::AppendRec\n");
#endif

  return AppendRec(pRecHeader->Count, pRecHeader->Comment.Get(), pData);
}

int DIGFileSnarkRec::AppendRec(unsigned int pCount, const char* pComment, double* pData)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::AppendRec\n");
#endif

  return AppendArray(pCount, pComment, pData);
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Reading
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Open for reading
///////////////////////////////////////////////////////////////////////////////

int DIGFileSnarkRec::Open(const char* FileName) 
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::Open\n");
#endif

  if(DIGFile::Open(FileName) != 0) {
#if DIGFILESNARKREC_DEBUG_LEVEL > 0
  printf("** Error: Openning DIG file\n");
#endif
    return -1; // error opening dig file
  }

  if(strcmp(MainHeader.Type.Get(), TypeStr) != 0) {
#if DIGFILESNARKREC_DEBUG_LEVEL > 0
  printf("** Error: Not a rsnark05 reffil file\n");
#endif
    return -2; // not a RecFile
  }

  // check if there is a phantom in the file

  // select first array set
  if(SelectArraySet(0) != 0) {
		return 4;
  }

  const char* type;

  if(GetArraySetType(&type) != 0) {
		return 6;
  }

  //// Move to first reconstruction array
  if(strncmp(type, "PHAN", 4) == 0) { // if first set is phantom set
    PhantomPresent = true;
  }
  else {
    PhantomPresent = false;
  }

  return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Reading main header 
///////////////////////////////////////////////////////////////////////////////

int DIGFileSnarkRec::GetMainHeader(RecFileMH* MainHeader)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::GetMainHeader\n");
#endif

  // read title
  const char* Title;

  if(GetTitle(&Title) != 0) {
    printf("Error reading Main Header Title\n");
    return -1;
  }

  printf("Main Header Title: %s\n", Title);

  if(MainHeader->Title.Set(Title) != 0) {
    printf("Error seting Main Header Title\n");
    return -1;
  }


  if(GetDimensions(&(MainHeader->Dimensions)) != 0) {
    printf("Error reading Main Header Dimensions\n");
    return -1;
  }

  printf("Main Header Dimensions: %d\n", MainHeader->Dimensions);

  if(GetSampling(&(MainHeader->Sampling)) != 0) {
    printf("Error reading Main Header Sampling\n");
    return -1;
  }

  printf("Main Header Sampling: %f\n", MainHeader->Sampling);


  const char* Comment;

  if(GetComment(&Comment) != 0) {
    printf("Error reading Main Header Comment\n");
    return -1;
  }

  printf("Main Header Comment: %s\n", Comment);

  if(MainHeader->Comment.Set(Comment) != 0) {
    printf("Error Seting Main Header Comment\n");
    return -1;
  }

  return 0;
}

///////////////////////////////////////////////////////////////////////////////
// Reading phantom header
///////////////////////////////////////////////////////////////////////////////

int DIGFileSnarkRec::GetPhantomHeader(RecFilePhanomH* pPhantomHeader)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::GetPhantomHeader\n");
#endif

  // read Name

  const char* Name;

  if(GetPhantomName(&Name) != 0) {
    printf("Error reading Phantom Name\n");
    return -1;
  }

  printf("Phantom Name: %s\n", Name);

  if(pPhantomHeader->Name.Set(Name) != 0) {
    printf("Error Setting Phantom Name\n");
    return -1;
  }

  // read comment

  const char* Comment;

  if(GetPhantomComment(&Comment) != 0) {
    printf("Error reading Phantom Comment\n");
    return -1;
  }

  printf("Phantom Comment: %s\n", Comment);

  if(pPhantomHeader->Comment.Set(Comment) != 0) {
    printf("Error Setting Phantom Comment\n");
    return -1;
  }

  return 0;
}

int DIGFileSnarkRec::GetPhantomName(const char** pPhantomName)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::GetPhantomName\n");
#endif

  if(!PhantomPresent) {

#if DIGFILESNARKREC_DEBUG_LEVEL > 0
  printf("** Error: No phantom in file\n");
#endif
    return -1; // no phantom in file 
  }

  // select first array set
  if(SelectArraySet(0) != 0) {

#if DIGFILESNARKREC_DEBUG_LEVEL > 0
  printf("** Error: No array sets in file\n");
#endif
		return -2;
  }

  return DIGFile::GetArraySetTitle(pPhantomName);
}

int DIGFileSnarkRec::GetPhantomComment(const char** pPhantomComment)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::GetPhantomComment\n");
#endif

  if(!PhantomPresent) {

#if DIGFILESNARKREC_DEBUG_LEVEL > 0
  printf("** Error: No phantom in file\n");
#endif
    return -1; // no phantom in file 
  }

  // select first array set
  if(SelectArraySet(0) != 0) {

#if DIGFILESNARKREC_DEBUG_LEVEL > 0
  printf("** Error: No phantom in file\n");
#endif
		return -2;
  }

  return DIGFile::GetArraySetComment(pPhantomComment);
}

///////////////////////////////////////////////////////////////////////////////
// Reading phantom data
///////////////////////////////////////////////////////////////////////////////

int DIGFileSnarkRec::GetPhantomData(void* pPhantomData)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::GetPhantomData\n");
#endif

  if(!PhantomPresent) {
    return -1; // no phantom in file 
  }

  // select first array set
  if(SelectArraySet(0) != 0) {
#if DIGFILESNARKREC_DEBUG_LEVEL > 0
  printf("** Error: No phantom in file\n");
#endif
		return -2;
  }

  // select first array
  if(SelectArray(0) != 0) {
#if DIGFILESNARKREC_DEBUG_LEVEL > 0
  printf("** Error: No phantom in file\n");
#endif
		return -3;
  }

  return DIGFile::GetArrayData(pPhantomData);
}

///////////////////////////////////////////////////////////////////////////////
// Reading reconstruction set header
///////////////////////////////////////////////////////////////////////////////

int DIGFileSnarkRec::GetRecSetHeader(RecFileRecSetH* pRecSetHeader)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::GetRecSetHeader\n");
#endif

  // read Name

  const char* Name;

  if(GetRecSetName(&Name) != 0) {
    printf("Error reading Reconstruction Name\n");
    return -1;
  }

  printf("Reconstruction Name: %s\n", Name);

  if(pRecSetHeader->Name.Set(Name) != 0) {
    printf("Error Setting Reconstruction Name\n");
    return -1;
  }

  // read AlgName

  const char* AlgName;

  if(GetRecSetAlgName(&AlgName) != 0) {
    printf("Error reading Reconstruction AlgName\n");
    return -1;
  }

  printf("Reconstruction Name: %s\n", AlgName);

  if(pRecSetHeader->Name.Set(AlgName) != 0) {
    printf("Error Setting Reconstruction AlgName\n");
    return -1;
  }

  // read comment

  const char* Comment;

  if(GetRecSetComment(&Comment) != 0) {
    printf("Error reading Reconstruction Set Comment\n");
    return -1;
  }

  printf("Reconstruction Comment: %s\n", Comment);

  if(pRecSetHeader->Comment.Set(Comment) != 0) {
    printf("Error Setting Reconstruction Set Comment\n");
    return -1;
  }

  return 0;
}

int DIGFileSnarkRec::GetRecSetName(const char** pRecName)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::GetRecSetName\n");
#endif

  return DIGFile::GetArraySetTitle(pRecName);
}

int DIGFileSnarkRec::GetRecSetAlgName(const char** pAlgName)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::GetRecSetAlgName\n");
#endif

  return DIGFile::GetArraySetType(pAlgName);
}

int DIGFileSnarkRec::GetRecSetComment(const char** pComment)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::GetRecSetComment\n");
#endif

  return DIGFile::GetArraySetComment(pComment);
}

///////////////////////////////////////////////////////////////////////////////
// Reading reconstruction header
///////////////////////////////////////////////////////////////////////////////

int DIGFileSnarkRec::GetRecHeader(RecFileRecH* RecHeader)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::GetRecHeader\n");
#endif

  // read Reconstruction Count

  if(GetRecCount(&(RecHeader->Count)) != 0) {
    printf("Error reading Reconstruction Count\n");
    return -1;
  }

  printf("Reconstruction Count: %u\n", RecHeader->Count);

  // read comment

  const char* Comment;

  if(GetRecComment(&Comment) != 0) {
    printf("Error reading Reconstruction Comment\n");
    return -1;
  }

  printf("Reconstruction Comment: %s\n", Comment);

  if(RecHeader->Comment.Set(Comment) != 0) {
    printf("Error Setting Reconstruction Comment\n");
    return -1;
  }

  return 0;
}


int DIGFileSnarkRec::GetRecCount(unsigned int* pCount)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::GetRecCount\n");
#endif

  return DIGFile::GetArrayEnumNo(pCount);
}

int DIGFileSnarkRec::GetRecComment(const char** pComment)
{
#if DIGFILESNARKREC_DEBUG_LEVEL > 3
  fprintf(stderr, "DIGFileSnarkRec::GetRecComment\n");
#endif

  return DIGFile::GetArrayComment(pComment);
}



