/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                              S N A R K   1 4                              *
 *                                                                           *
 *                     A PICTURE RECONSTRUCTION PROGRAM                      *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

 snfft.cpp,v 1.2 2008/08/25 16:11:09 jklukowska Exp

    SUBROUTINE SNFFT
    PURPOSE
       PERFORMS DISCRETE COMPLEX FOURIER TRANSFORMS ON A COMPLEX
       THREE-DIMENSIONAL ARRAY.
    USAGE
       CALL SNFFT(X,N,INVDIR)
    DESCRIPTION
       X     - AS INPUT, X CONTAINS THE COMPLEX THREE-DIMENSIONAL
               ARRAY TO BE TRANSFORMED. THE REAL PART OF X(K,L,M)
               IS STORED IN VECTOR FASHION IN A CELL WITH INDEX
               2*(K*N(3)*N(2)+L*N(3)+M)+1, THE IMAGINARY PART IS
               STORED IN THE CELL IMMEDIATELY FOLLOWING. NOTE THAT
               THE SUBSCRIPT K INCREASES MOST RAPIDLY, AND M LEAST
               RAPIDLY.
               AS OUTPUT, X CONTAINS THE COMPLEX FOURIER TRANSFORM
               OR THE INVERSE TRANSFORM, DEPENDING ON INVDIR.
       N     - A THREE CELL VECTOR WHICH DETERMINES THE SIZE OF
               THE THREE DIMENSIONS OF X.
       INVDIR- INTEGER, WHICH SHOULD BE EITHER +1 OR -1, WHICH
               DETERMINES WHETHER THE DIRECT OR INVERSE TRANSFORM
               IS TO BE PERFORMED.
    REMARKS
       THIS SUBROUTINE CAN BE USED IN A SNARK-ENVIRONMENT ONLY]
       THE WORKSPACE THIS SUBROUTINE REQUIRES IS ALLOCATED FROM
       THE BLANK COMMON BLOCK, USING THE SNARK SUBROUTINE ALLOC.
       THIS WORKSPACE IS RELEASED UPON EXIT OF SNFFT.
       THE SUBROUTINE CAN BE USED FOR ANY VALUES OF THE VECTOR N.
       IT IS POSSIBLE THAT THE SUBROUTINE CAUSES A HALT , DUE TO
       THE FACT THAT NOT ENOUGH SPACE IN BLANK COMMON IS AVAILABLE.
    SUBROUTINES REQUIRED
       FFT2P, FTODD, UNKPS, FREE
    FUNCTIONS REQUIRED
       ALLOC, MAVIRT
       INTEGER FUNCTION MAVIRT(I,J,Y)
       INTEGER I,J
       REAL Y(1)
       THIS FUNCTION IS ONLY REQUIRED DURING LOADING,BUT IS NOT
       CALLED DURING EXECUTION.
    SOURCE
       THE SUBROUTINES FFT2P, FTODD AND UNKPS ARE FROM
          POLGE,R.J. AND BHAGAVAN,B.K.,EFFICIENT FAST FOURIER
               TRANSFORM PROGRAMS FOR ARBITRARY FACTORS WITH
               ONE STEP LOOP UNSCRAMBLING. IEEE TRANSACTIONS ON
               COMPUTERS, MAY 1976, PP. 534-539.
    METHOD
       FOR INVDIR = +1 THE FORIER TRANSFORM OF COMPLEX ARRAY
       X IS OBTAINED.

           N(1)-1  N(2)-1  N(3)-1            -L1  -L2  -L3
  X(K,L,M)=SUM     SUM     SUM     X(P,Q,R)*W1  *W2  *W3
           P=0     Q=0     R=0

       WHERE WI IS THE N(I)-TH ROOT OF UNITY AND L1=P*K,
       L2=L*Q AND L3=R*M.
       FOR INVDIR = -1 THE INVERSE TRANSFORM  OF COMPLEX ARRAY
       X IS OBTAINED.

 X(K,L,M) =

    1      N(1)-1  N(2)-1  N(3)-1             L1   L2   L3
   ---- *  SUM     SUM     SUM     X(P,Q,R)*W1  *W2  *W3
   NORM    P=0     Q=0     R=0

       WHERE NORM = N(1)*N(2)*N(3).
 ..................................................................
*/

#include <cstdlib>
#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "uiod.h"

#include "primfc.h"
#include "fft2p.h"
#include "snfft.h"    


void snfft(REAL* x, INTEGER* n, INTEGER invdir)
{  
  INTEGER i;
  INTEGER nxtelt;
  INTEGER upfac;
  INTEGER ifft;
  INTEGER dostep;
  INTEGER uppbnd;
  INTEGER* ipf;
  INTEGER m;
  INTEGER nipf;
  INTEGER ipivot;
  INTEGER ndftau;
  INTEGER ipusiz;
  INTEGER itsize;
  //INTEGER* ind;
  INTEGER ind1;
  INTEGER ixsiz;
  REAL* ivspt = NULL;
  REAL* it = NULL;
  INTEGER* ix = NULL;
  INTEGER* ipu = NULL;
  REAL* iftaux = NULL;    //wei 3/2005
  INTEGER j;
  INTEGER lhi;
  REAL facnor;
  INTEGER l;

  if(abs(invdir) != 1)
  {
    fprintf(output, "\n **** parameter invdir of function snfft() should be +1 or -1");
    fprintf(output, "\n **** program aborted\n");
    exit(0);
  }

  for(i = 0; i < 3; i++) {
    if(n[i] < 1)
    {
      fprintf(output, "\n **** parameter n[%d] of function snfft() should be positive", i);
      fprintf(output, "\n **** program aborted\n");
      exit(0);
    }
  }

  nxtelt = 1;
  upfac = n[0] * n[1] * n[2];
  for(i = 0; i < 3; i++) {

    ifft = n[2-i];

    if(ifft != 1) {
      upfac /= ifft;
      dostep = nxtelt * ifft;
      uppbnd = dostep * (upfac - 1) + 1;

      ipf = new INTEGER[7];

      primfc(ifft, &m, &nipf, ipf, 7);

      ipivot = ifft / (1 << (2 * (m / 2)));

      ndftau = 2 * nxtelt;

      ipusiz = (3 * ipivot) / 2 + 2;

      itsize = 2;

      if(nipf != 0) {
        ndftau = 2 * ipf[nipf - 1] * nxtelt;
        itsize = 2 * ipf[nipf - 1];
      }
      ixsiz = MAX0(ipusiz, (1 << (m/2)));
      ivspt = new REAL[itsize];
      it = new REAL[itsize];
      ix = new INTEGER[ixsiz];
      ipu = new INTEGER[ipusiz];
      iftaux = new REAL[ndftau];
      for (j=0;j<itsize;j++) ivspt[j]=0;
      for (j=0;j<itsize;j++) it[j]=0;
      for (j=0;j<ixsiz;j++) ix[j]=0;
      for (j=0;j<ipusiz;j++) ipu[j]=0;
      for (j=0;j<ndftau;j++) iftaux[j]=0;

      for(j = 0; j < uppbnd; j += dostep) {
        ind1 = 2 * j;
        fft2p(nxtelt, nxtelt, iftaux, x+ind1, m, -invdir, ix, nipf, ipf, ipu, ivspt, it, FALSE);

        // test /////////////////////////////////////////
        /*
        fprintf(output,"\n snfft ivspt");
        for(int xx = 0; xx < itsize; xx++) {
          if((xx % 7) == 0) fprintf(output,"\n");
          fprintf(output," %9.4f", ivspt[xx]);
        }
        */
        /*
        fprintf(output,"\n snfft it");
        for(int xx = 0; xx < itsize; xx++) {
          if((xx % 7) == 0) fprintf(output,"\n");
          fprintf(output," %9.4f", it[xx]);
        }
        */

        /*
        fprintf(output,"\n snfft x");
        for(int xx = 0; xx < 2 * n[0] * n[1] * n[2]; xx++) {
          if((xx % 7) == 0) fprintf(output,"\n");
          fprintf(output," %9.4f", x[xx]);
        }
        */
        /////////////////////////////////////////////////
      }

      //////////////////////////////
      /*
        fprintf(output,"\n snfft x");
        for(int xx = 0; xx < 2 * n[0] * n[1] * n[2]; xx++) {
          if((xx % 7) == 0) fprintf(output,"\n");
          fprintf(output," %9.4f", x[xx]);
        }
      */
      /////////////////////////////////////////////////////

      nxtelt *= ifft;

      delete[] ipf;  // bug 92 - Lajos - 03/02/2005

    }
  }

  if(invdir == -1) {

    lhi = 2 * n[0] * n[1] * n[2];

    facnor = ((REAL) 2.0) / ((REAL) lhi);

    for(l = 0; l < lhi; l++) {
      x[l] *= facnor;
    }
  }


       if(ivspt != NULL)  delete[] ivspt;
       if(it != NULL)  delete[] it;   
       if(ix != NULL)  delete[] ix;
       if(ipu != NULL)  delete[] ipu;
       if(iftaux != NULL)  delete[] iftaux;       //wei 3/2005
}      
