/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                              S N A R K   1 4                              *
 *                                                                           *
 *                     A PICTURE RECONSTRUCTION PROGRAM                      *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

 subreg.cpp,v 1.2 2008/08/25 16:11:09 jklukowska Exp

 Subroutine to compute rough boundary (a bounding rectangle) of 
 the specified region to reduce the search area that requires
 within region test.
 Specified region is either a rectangle or an ellipse.
 The bounding rectangle is the SMALLEST rectangle, with the same
 orientation as the picture, that bounds the nominal rectangle defined
 to have the same origin, axes and orientation as the specified region
 If this bounding rectangle (AN ESTIMATION) is partially outside the 
 picture area, a WARNING message is output and the bounding rectangle
 is clipped to the picture boundary.
 This bounding rectangle is then passed to the DIST routine where a 
 rigorous within boundary test reduced to this area is performed.

 Parameters are passed through COMMON /region/
*/


// parameter (MXNREG=40)

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "region.h"
#include "uiod.h"
#include "consts.h"

#include "eval.h"
 
void Eval_class::subreg(INTEGER ObjJ)
{
  REAL cx;
  REAL cy;
  REAL a;
  REAL b;

  REAL theta;
  REAL cs;
  REAL ss;

  INTEGER ixw;
  INTEGER iyw;
  INTEGER ic;
  INTEGER icx;
  INTEGER icy;

  REAL tx1;
  REAL tx2;
  REAL ty1;
  REAL ty2;

  REAL xw;
  REAL yw;

  REAL x0;
  REAL y0;


  REAL xs;
  REAL ys;

  static BOOLEAN flgout;
  //  static BOOLEAN objdon[MXNREG];  moved to eval.h - bug112 - swr - 6/10/05

  BOOLEAN idone;

  // Set up parameters

  cx = Obj[ObjJ].cx;
  cy = Obj[ObjJ].cy;
  a = Obj[ObjJ].u;
  b = Obj[ObjJ].v;
  Region.aplus = a + (REAL) 0.0001;
  Region.bplus = b + (REAL) 0.0001;
  theta = Obj[ObjJ].ang * (REAL) Consts.pi/180.0;
  cs = (REAL) cos(theta);
  ss = (REAL) sin(theta);

  Region.d1 = GeoPar.pixsiz * cs;
  Region.d2 = GeoPar.pixsiz * ss;
  Region.asq = SQR(a);
  Region.bsq = SQR(b);

  // Estimate boundary of region

  tx1 = (REAL) fabs(a * cs + b * ss);
  tx2 = (REAL) fabs(a * cs - b * ss);
  ty1 = (REAL) fabs(a * ss - b * cs);
  ty2 = (REAL) fabs(a * ss + b * cs);
  xw = MAX0(tx1, tx2);
  yw = MAX0(ty1, ty2);
  ixw = lround(xw / GeoPar.pixsiz);  // changed "(INTEGER)" to "lround" to match Fortran code. Lajos, Jan 20, 2005
  iyw = lround(yw / GeoPar.pixsiz);  // changed "(INTEGER)" to "lround" to match Fortran code. Lajos, Jan 20, 2005
  ic = GeoPar.nelem / 2;
  icx = ic + lround(cx / GeoPar.pixsiz);  // changed "(INTEGER)" to "lround" to match Fortran code. Lajos, Jan 20, 2005
  icy = ic - lround(cy / GeoPar.pixsiz);  // changed "(INTEGER)" to "lround" to match Fortran code. Lajos, Jan 20, 2005

  Region.ix1 = icx - ixw;
  Region.ix2 = icx + ixw;
  Region.iy1 = icy - iyw;
  Region.iy2 = icy + iyw;

  // Ensure estimated boundary is within picture region

  idone = objdon[ObjJ];
  flgout = FALSE;
  if(Region.ix1 < 0) {

    if((!flgout) && (idone == 0)) {
      fprintf(output, "\n         WARNING: Region %2i might not be entirely within picture", ObjJ);
      fprintf(output, "\n         Rough estimate of object boundary:");
      fprintf(output, "\n         ix1     ix2     iy1     iy2");
      fprintf(output, "\n         %3i     %3i     %3i     %3i", Region.ix1, Region.ix2, Region.iy1, Region.iy2);
    }

    flgout = TRUE;
    Region.ix1 = 0;
  }

  if(Region.ix2 > GeoPar.nelem - 1) {
    if((!flgout) && (idone == 0)) {
      fprintf(output, "\n         WARNING: Region %2i might not be entirely within picture", ObjJ);
      fprintf(output, "\n         Rough estimate of object boundary:");
      fprintf(output, "\n         ix1     ix2     iy1     iy2");
      fprintf(output, "\n         %3i     %3i     %3i     %3i", Region.ix1, Region.ix2, Region.iy1, Region.iy2);
    }
    flgout = TRUE;
    Region.ix2 = GeoPar.nelem - 1;
  }

  if(Region.iy1 < 0) {
    if((!flgout) && (idone == 0)) {
      fprintf(output, "\n         WARNING: Region %2i might not be entirely within picture", ObjJ);
      fprintf(output, "\n         Rough estimate of object boundary:");
      fprintf(output, "\n         ix1     ix2     iy1     iy2");
      fprintf(output, "\n         %3i     %3i     %3i     %3i", Region.ix1, Region.ix2, Region.iy1, Region.iy2);
    }
    flgout = TRUE;
    Region.iy1 = 0;
  }

  if(Region.iy2 > GeoPar.nelem - 1) {
    if((!flgout) && (idone == 0)) {
      fprintf(output, "\n         WARNING: Region %2i might not be entirely within picture", ObjJ);
      fprintf(output, "\n         Rough estimate of object boundary:");
      fprintf(output, "\n         ix1     ix2     iy1     iy2");
      fprintf(output, "\n         %3i     %3i     %3i     %3i", Region.ix1, Region.ix2, Region.iy1, Region.iy2);
    }
    flgout = TRUE;
    Region.iy2 = GeoPar.nelem - 1;
  }

  objdon[ObjJ] = true;

  x0 = (Region.ix1 - ic) * GeoPar.pixsiz;
  y0 = (ic - Region.iy1) * GeoPar.pixsiz;
  xs = x0 - cx;
  ys = y0 - cy;
  Region.xd0 = xs * cs + ys * ss;
  Region.yd0 = -xs * ss + ys * cs;
}
